#!/usr/bin/env python
# -*- coding:utf-8 -*-

"""
@author:    george wang
@datetime:  2019-05-11
@file:      exception.py
@contact:   georgewang1994@163.com
@desc:      错误
"""


class NotRegisterException(Exception):
    pass
