#!/usr/bin/env python
# -*- coding:utf-8 -*-

"""
@author:    george wang
@datetime:  2019-06-30
@file:      ammonia.py
@contact:   georgewang1994@163.com
@desc:      ...
"""

from ammonia.app import Ammonia


ammonia = Ammonia()
